﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Datenaufbewahrung                                                                   //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright © 2010 Reimar Grasbon (softwork@grasbon.de)                               //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "Conf.h"
#include "TypeDefs.h"
#include "Settings.h"
#include "DateTime.h"
#include "Alarm.h"
#include "Application.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define STO_ATTR_OFFSET(name)        ((UInt16)&factoryDef.name-(UInt16)&factoryDef)
#define STO_ATTR_SIZE(name)          (sizeof(factoryDef.name))
#define STORING_Load(name, value)    STORING_LoadEEPMem(STO_ATTR_OFFSET(name), value, STO_ATTR_SIZE(name))
#define STORING_Save(name, value)    STORING_SaveEEPMem(STO_ATTR_OFFSET(name), value, STO_ATTR_SIZE(name))
#define STORING_Default(name, value) STORING_LoadPGMMem(STO_ATTR_OFFSET(name), value, STO_ATTR_SIZE(name))

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef enum       // Deutsche Bundesländer:
{   GR_BW =  0,    // · Baden-Württemberg
    GR_BY =  1,    // · Bayern
    GR_BE =  2,    // · Berlin
    GR_BB =  3,    // · Brandenburg
    GR_HB =  4,    // · Bremen
    GR_HH =  5,    // · Hamburg
    GR_HE =  6,    // · Hessen
    GR_MV =  7,    // · Mecklenburg-Vorpommern
    GR_NI =  8,    // · Niedersachsen
    GR_NW =  9,    // · Nordrhein-Westfalen
    GR_RP = 10,    // · Rheinland-Pfalz
    GR_SL = 11,    // · Saarland
    GR_SN = 12,    // · Sachsen
    GR_ST = 13,    // · Sachsen-Anhalt
    GR_SH = 14,    // · Schleswig-Holstein
    GR_TH = 15     // · Thüringen
} GermanRegion;

typedef enum            // Sommerzeit-Einstellungen
{   DSM_OFF  =  0,      // · Aus: Normalzeit
    DSM_ON   =  1,      // · Ein: Sommerzeit
    DSM_AUTO =  2       // · Automatische Umschaltung
} DaylightSavingMode;

typedef struct  	// Wertespeicherstruktur
{   UInt08          STO_DisplayBlankStart;
    UInt08          STO_MotionSensDisplay;
    bool            STO_AlarmEnabled;
    AlarmOperation  STO_AlarmOperation;
    UInt08          STO_AlarmDuration;
    Time            STO_AlarmTime;
    bool            STO_AlarmTypeRadio;
    bool            STO_SnoozeEnabled;
    UInt08          STO_MotionSensSnooze;
    UInt08          STO_SnoozeDuration;
    GermanRegion    STO_GermanRegion;
    bool            STO_ShowHoliday;
    DaylightSavingMode  STO_DaylightSavingMode;
} Storing;

//////////////////////////////////////////////////////////////////////////////////////////
//  Konstanten  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
extern const Storing  factoryDef;

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void    STORING_LoadEEPMem(UInt16, void*      , UInt16);
void    STORING_SaveEEPMem(UInt16, const void*, UInt16);
void    STORING_LoadPGMMem(UInt16, void*      , UInt16);
void    STORING_RestoreDefaults(void);

//////////////////////////////////////////////////////////////////////////////////////////
//  System  //////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void    STORING_Startup(void);
void    STORING_Proceed(void);
void    STORING_Shutdown(void);

//////////////////////////////////////////////////////////////////////////////////////////
